@@ -10,53 +10,20 @@
 // headers
 //-----------------------------------------------------------------------------
 
-#ifdef __VMS
-	#define __USE_STD_IOSTREAM 1
-	#include <assert>
-#else
-	#include <cassert>
-#endif
+#include <cassert>
 
 #include <cctype>
-#include <iostream>		// for cout
+#include <iostream>
 #include <memory>
 #include <string>
+#include <string_view>
 #include <vector>
 
 #ifdef __GNUC__
 	#include <cstring>              // need both string and cstring for GCC
 #endif
 
-//-----------------------------------------------------------------------------
-// declarations
-//-----------------------------------------------------------------------------
-
-#ifdef _MSC_VER
-	#pragma warning(disable: 4267)  // conversion from size_t to int
-#endif
-
-#ifdef __BORLANDC__
-	#pragma warn -8004	            // variable is assigned a value that is never used
-#endif
-
-#ifdef __GNUC__
-	#pragma GCC diagnostic ignored "-Wconversion"
-#endif
-
-#ifdef __INTEL_COMPILER
-	// #pragma warning disable 383  // value copied to temporary, reference to temporary used
-	// #pragma warning disable 981  // operands are evaluated in unspecified order
-#endif
-
-#ifdef __clang__
-	#pragma clang diagnostic ignored "-Wshorten-64-to-32"
-#endif
-
-#define ASTYLE_VERSION "3.4.9"
-
-//-----------------------------------------------------------------------------
-// astyle namespace
-//-----------------------------------------------------------------------------
+#define ASTYLE_VERSION "3.6.1"
 
 namespace astyle {
 
@@ -64,7 +31,7 @@ namespace astyle {
 // definitions
 //----------------------------------------------------------------------------
 
-enum FileType { C_TYPE = 0, JAVA_TYPE = 1, SHARP_TYPE = 2, JS_TYPE = 3, OBJC_TYPE = 4, INVALID_TYPE = -1 };
+enum FileType { C_TYPE = 0, JAVA_TYPE = 1, SHARP_TYPE = 2, JS_TYPE = 3, OBJC_TYPE = 4, GSC_TYPE = 5, INVALID_TYPE = -1 };
 
 /* The enums below are not recognized by 'vectors' in Microsoft Visual C++
    V5 when they are part of a namespace!!!  Use Visual C++ V6 or higher.
@@ -169,10 +136,21 @@ enum LineEndFormat
 	LINEEND_DEFAULT,	// Use line break that matches most of the file
 	LINEEND_WINDOWS,
 	LINEEND_LINUX,
-	LINEEND_MACOLD,
-	LINEEND_CRLF = LINEEND_WINDOWS,
-	LINEEND_LF   = LINEEND_LINUX,
-	LINEEND_CR   = LINEEND_MACOLD
+	LINEEND_MACOLD
+};
+
+enum NegationPaddingMode
+{
+	NEGATION_PAD_NO_CHANGE,
+	NEGATION_PAD_AFTER,
+	NEGATION_PAD_BEFORE
+};
+
+enum IncludeDirectivePaddingMode
+{
+	INCLUDE_PAD_NO_CHANGE,
+	INCLUDE_PAD_NONE,
+	INCLUDE_PAD_AFTER
 };
 
 //-----------------------------------------------------------------------------
@@ -265,6 +243,7 @@ public:
 	static const std::string AS_OPEN_PAREN, AS_CLOSE_PAREN;
 	static const std::string AS_OPEN_BRACE, AS_CLOSE_BRACE;
 	static const std::string AS_OPEN_LINE_COMMENT, AS_OPEN_COMMENT, AS_CLOSE_COMMENT;
+	static const std::string AS_GSC_OPEN_COMMENT, AS_GSC_CLOSE_COMMENT;
 	static const std::string AS_BAR_DEFINE, AS_BAR_INCLUDE, AS_BAR_IF, AS_BAR_EL, AS_BAR_ENDIF;
 	static const std::string AS_AUTO, AS_RETURN;
 	static const std::string AS_CIN, AS_COUT, AS_CERR, AS_MAPPING;
@@ -305,28 +284,28 @@ protected:
 
 protected:  // inline functions
 	void init(int fileTypeArg) { baseFileType = fileTypeArg; }
-	bool isCStyle() const { return baseFileType == C_TYPE || baseFileType == OBJC_TYPE; }
+	bool isCStyle() const { return baseFileType == C_TYPE || baseFileType == OBJC_TYPE || baseFileType == GSC_TYPE; }
 	bool isJavaStyle() const { return baseFileType == JAVA_TYPE; }
 	bool isSharpStyle() const { return baseFileType == SHARP_TYPE; }
 	bool isJSStyle() const { return baseFileType == JS_TYPE; }
 	bool isObjCStyle() const { return baseFileType == OBJC_TYPE; }
+	bool isGSCStyle() const { return baseFileType == GSC_TYPE; }
 
 	bool isWhiteSpace(char ch) const { return (ch == ' ' || ch == '\t'); }
 
 protected:  // functions definitions are at the end of ASResource.cpp
-	const std::string* findHeader(const std::string& line, int i,
+	const std::string* findHeader(std::string_view line, int i,
 	                              const std::vector<const std::string*>* possibleHeaders) const;
-	bool findKeyword(const std::string& line, int i, const std::string& keyword) const;
-	const std::string* findOperator(const std::string& line, int i,
+	bool findKeyword(std::string_view  line, int i, std::string_view  keyword) const;
+	const std::string* findOperator(std::string_view  line, int i,
 	                                const std::vector<const std::string*>* possibleOperators) const;
-	std::string getCurrentWord(const std::string& line, size_t index) const;
+	std::string_view getCurrentWord(std::string_view, size_t index) const;
 	bool isDigit(char ch) const;
 	bool isLegalNameChar(char ch) const;
-	bool isCharPotentialHeader(const std::string& line, size_t i) const;
+	bool isCharPotentialHeader(std::string_view line, size_t i) const;
 	bool isCharPotentialOperator(char ch) const;
-	bool isDigitSeparator(const std::string& line, int i) const;
-	char peekNextChar(const std::string& line, int i) const;
-
+	bool isDigitSeparator(std::string_view line, int i) const;
+	char peekNextChar(std::string_view line, int i) const;
 };  // Class ASBase
 
 //-----------------------------------------------------------------------------
@@ -357,6 +336,7 @@ public:
 	void setJSStyle();
 	void setObjCStyle();
 	void setSharpStyle();
+	void setGSCStyle();
 
 	void setLabelIndent(bool state);
 	void setMaxContinuationIndentLength(int max);
@@ -395,14 +375,14 @@ public:
 
 protected:
 	void deleteBeautifierVectors();
-	int  getNextProgramCharDistance(const std::string& line, int i) const;
+	int  getNextProgramCharDistance(std::string_view line, int i) const;
 	int  indexOf(const std::vector<const std::string*>& container, const std::string* element) const;
 	void setBlockIndent(bool state);
 	void setBraceIndent(bool state);
 	void setBraceIndentVtk(bool state);
-	std::string extractPreprocessorStatement(const std::string& line) const;
-	std::string trim(const std::string& str) const;
-	std::string rtrim(const std::string& str) const;
+	std::string extractPreprocessorStatement(std::string_view line) const;
+	std::string trim(std::string_view str) const;
+	std::string rtrim(std::string_view str) const;
 
 	// variables set by ASFormatter - must be updated in activeBeautifierStack
 	int  inLineNumber;
@@ -423,16 +403,16 @@ protected:
 
 
 private:  // functions
-	void adjustObjCMethodDefinitionIndentation(const std::string& line_);
-	void adjustObjCMethodCallIndentation(const std::string& line_);
+	void adjustObjCMethodDefinitionIndentation(std::string_view line_);
+	void adjustObjCMethodCallIndentation(std::string_view line_);
 	void adjustParsedLineIndentation(size_t iPrelim, bool isInExtraHeaderIndent);
 	void computePreliminaryIndentation();
-	void parseCurrentLine(const std::string& line);
+	void parseCurrentLine(std::string_view line);
 	void popLastContinuationIndent();
-	void processPreprocessor(const std::string& preproc, const std::string& line);
-	void registerContinuationIndent(const std::string& line, int i, int spaceIndentCount_,
+	void processPreprocessor(std::string_view preproc, std::string_view line);
+	void registerContinuationIndent(std::string_view line, int i, int spaceIndentCount_,
 	                                int tabIncrementIn, int minIndent, bool updateParenStack);
-	void registerContinuationIndentColon(const std::string& line, int i, int tabIncrementIn);
+	void registerContinuationIndentColon(std::string_view line, int i, int tabIncrementIn);
 	void initVectors();
 	void initTempStacksContainer(std::vector<std::vector<const std::string*>*>*& container,
 	                             std::vector<std::vector<const std::string*>*>* value);
@@ -440,20 +420,20 @@ private:  // functions
 	void deleteBeautifierContainer(std::vector<ASBeautifier*>*& container);
 	void deleteTempStacksContainer(std::vector<std::vector<const std::string*>*>*& container);
 	int  adjustIndentCountForBreakElseIfComments() const;
-	int  computeObjCColonAlignment(const std::string& line, int colonAlignPosition) const;
+	int  computeObjCColonAlignment(std::string_view line, int colonAlignPosition) const;
 	int  convertTabToSpaces(int i, int tabIncrementIn) const;
-	int  findObjCColonAlignment(const std::string& line) const;
-	int  getContinuationIndentAssign(const std::string& line, size_t currPos) const;
-	int  getContinuationIndentComma(const std::string& line, size_t currPos) const;
-	int  getObjCFollowingKeyword(const std::string& line, int bracePos) const;
-	bool isIndentedPreprocessor(const std::string& line, size_t currPos) const;
-	bool isLineEndComment(const std::string& line, int startPos) const;
-	bool isPreprocessorConditionalCplusplus(const std::string& line) const;
-	bool isInPreprocessorUnterminatedComment(const std::string& line);
+	int  findObjCColonAlignment(std::string_view line) const;
+	int  getContinuationIndentAssign(std::string_view line, size_t currPos) const;
+	int  getContinuationIndentComma(std::string_view line, size_t currPos) const;
+	int  getObjCFollowingKeyword(std::string_view line, int bracePos) const;
+	bool isIndentedPreprocessor(std::string_view line, size_t currPos) const;
+	bool isLineEndComment(std::string_view line, int startPos) const;
+	bool isPreprocessorConditionalCplusplus(std::string_view line) const;
+	bool isInPreprocessorUnterminatedComment(std::string_view line);
 	bool isTopLevel() const;
-	bool statementEndsWithComma(const std::string& line, int index) const;
-	const std::string& getIndentedLineReturn(const std::string& newLine, const std::string& originalLine) const;
-	std::string getIndentedSpaceEquivalent(const std::string& line_) const;
+	bool statementEndsWithComma(std::string_view line, int index) const;
+
+	std::string getIndentedSpaceEquivalent(std::string_view line_) const;
 	std::string preLineWS(int lineIndentCount, int lineSpaceIndentCount) const;
 	template<typename T> void deleteContainer(T& container);
 	template<typename T> void initContainer(T& container, T value);
@@ -561,6 +541,11 @@ private:  // variables
 	bool shouldIndentPreprocConditional;
 	bool squeezeWhitespace;
 	bool attemptLambdaIndentation;
+
+	bool isInAssignment;
+	bool isInInitializerList;
+	bool isInMultiLineString;
+
 	int  indentCount;
 	int  spaceIndentCount;
 	int  spaceIndentObjCMethodAlignment;
@@ -595,6 +580,8 @@ private:  // variables
 // Class ASEnhancer
 //-----------------------------------------------------------------------------
 
+// TODO rewrite methods to return altered stings
+
 class ASEnhancer : protected ASBase
 {
 public:  // functions
@@ -606,11 +593,11 @@ public:  // functions
 private:  // functions
 	void   convertForceTabIndentToSpaces(std::string&  line) const;
 	void   convertSpaceIndentToForceTab(std::string& line) const;
-	size_t findCaseColon(const std::string&  line, size_t caseIndex) const;
+	size_t findCaseColon(std::string_view line, size_t caseIndex) const;
 	int    indentLine(std::string&  line, int indent) const;
-	bool   isBeginDeclareSectionSQL(const std::string&  line, size_t index) const;
-	bool   isEndDeclareSectionSQL(const std::string&  line, size_t index) const;
-	bool   isOneLineBlockReached(const std::string& line, int startChar) const;
+	bool   isBeginDeclareSectionSQL(std::string_view  line, size_t index) const;
+	bool   isEndDeclareSectionSQL(std::string_view  line, size_t index) const;
+	bool   isOneLineBlockReached(std::string_view line, int startChar) const;
 	void   parseCurrentLine(std::string& line, bool isInPreprocessor, bool isInSQL);
 	size_t processSwitchBlock(std::string&  line, size_t index);
 	int    unindentLine(std::string&  line, int unindent) const;
@@ -685,7 +672,7 @@ public:	// functions
 	LineEndFormat getLineEndFormat() const;
 	bool getIsLineReady() const;
 	void setFormattingStyle(FormatStyle style);
-	void setAddBracesMode(bool state);
+	void setAddBracesMode(int state);
 	void setAddOneLineBracesMode(bool state);
 	void setRemoveBracesMode(bool state);
 	void setAttachClass(bool state);
@@ -720,6 +707,10 @@ public:	// functions
 	void setMaxCodeLength(int max);
 	void setObjCColonPaddingMode(ObjCColonPad mode);
 	void setOperatorPaddingMode(bool state);
+	void setNegationPaddingMode(NegationPaddingMode mode);
+	void setIncludeDirectivePaddingMode(IncludeDirectivePaddingMode mode);
+
+
 	void setParensOutsidePaddingMode(bool state);
 	void setParensFirstPaddingMode(bool state);
 
@@ -757,8 +748,8 @@ private:  // functions
 	char peekNextChar() const;
 	BraceType getBraceType();
 	bool adjustChecksumIn(int adjustment);
-	bool computeChecksumIn(const std::string& currentLine_);
-	bool computeChecksumOut(const std::string& beautifiedLine);
+	bool computeChecksumIn(std::string_view currentLine_);
+	bool computeChecksumOut(std::string_view beautifiedLine);
 	bool addBracesToStatement();
 	bool removeBracesFromStatement();
 	bool commentAndHeaderFollows();
@@ -774,22 +765,22 @@ private:  // functions
 	bool isClosingHeader(const std::string* header) const;
 	bool isCurrentBraceBroken() const;
 	bool isDereferenceOrAddressOf() const;
-	bool isExecSQL(const std::string& line, size_t index) const;
-	bool isEmptyLine(const std::string& line) const;
+	bool isExecSQL(std::string_view line, size_t index) const;
+	bool isEmptyLine(std::string_view line) const;
 	bool isExternC() const;
 	bool isMultiStatementLine() const;
 	bool isNextWordSharpNonParenHeader(int startChar) const;
 	bool isNonInStatementArrayBrace() const;
-	bool isNumericVariable(const std::string& word) const;
+	bool isNumericVariable(std::string_view word) const;
 	bool isOkToSplitFormattedLine();
 	bool isPointerOrReference() const;
 	bool isPointerOrReferenceCentered() const;
-	bool isPointerOrReferenceVariable(const std::string& word) const;
-	bool isPointerToPointer(const std::string& line, int currPos) const;
+	bool isPointerOrReferenceVariable(std::string_view word) const;
+	bool isPointerToPointer(std::string_view line, int currPos) const;
 	bool isSharpStyleWithParen(const std::string* header) const;
 	bool isStructAccessModified(const std::string& firstLine, size_t index) const;
 	bool isIndentablePreprocessorBlock(const std::string& firstLine, size_t index);
-	bool isNDefPreprocStatement(const std::string& nextLine_, const std::string& preproc) const;
+	bool isNDefPreprocStatement(std::string_view nextLine_, std::string_view preproc) const;
 	bool isUnaryOperator() const;
 	bool isUniformInitializerBrace() const;
 	bool isImmediatelyPostCast() const;
@@ -802,19 +793,19 @@ private:  // functions
 	int  findObjCColonAlignment() const;
 	int  getCurrentLineCommentAdjustment();
 	int  getNextLineCommentAdjustment();
-	int  isOneLineBlockReached(const std::string& line, int startChar) const;
+	int  isOneLineBlockReached(std::string_view line, int startChar) const;
 	void adjustComments();
 	void appendChar(char ch, bool canBreakLine);
 	void appendCharInsideComments();
 	void appendClosingHeader();
-	void appendOperator(const std::string& sequence, bool canBreakLine = true);
-	void appendSequence(const std::string& sequence, bool canBreakLine = true);
+	void appendOperator(std::string_view sequence, bool canBreakLine = true);
+	void appendSequence(std::string_view sequence, bool canBreakLine = true);
 	void appendSpacePad();
 	void appendSpaceAfter();
 	void breakLine(bool isSplitLine = false);
 	void buildLanguageVectors();
 	void updateFormattedLineSplitPoints(char appendedChar);
-	void updateFormattedLineSplitPointsOperator(const std::string& sequence);
+	void updateFormattedLineSplitPointsOperator(std::string_view sequence);
 	void checkIfTemplateOpener();
 	void clearFormattedLineSplitPoints();
 	void convertTabToSpaces();
@@ -847,7 +838,7 @@ private:  // functions
 	void padObjCParamType();
 	void padObjCReturnType();
 	void padOperators(const std::string* newOperator);
-	void padParensOrBrackets(char openDelim, char closeDelim, bool shouldPadParensOutside, bool shouldPadParensInside, bool shouldUnPadParens, bool shouldPadFirstParen);
+	void padParensOrBrackets(char openDelim, char closeDelim, bool padFirstParen);
 	void processPreprocessor();
 	void resetEndOfStatement();
 	void setAttachClosingBraceMode(bool state);
@@ -856,11 +847,11 @@ private:  // functions
 	void trimContinuationLine();
 	void updateFormattedLineSplitPointsPointerOrReference(size_t index);
 	size_t findFormattedLineSplitPoint() const;
-	size_t findNextChar(const std::string& line, char searchChar, int searchStart = 0) const;
-	const std::string* checkForHeaderFollowingComment(const std::string& firstLine) const;
+	size_t findNextChar(std::string_view line, char searchChar, int searchStart = 0) const;
+	const std::string* checkForHeaderFollowingComment(std::string_view firstLine) const;
 	const std::string* getFollowingOperator() const;
-	std::string getPreviousWord(const std::string& line, int currPos) const;
-	std::string peekNextText(const std::string& firstLine,
+	std::string getPreviousWord(const std::string& line, int currPos, bool allowDots = false) const;
+	std::string peekNextText(std::string_view firstLine,
 	                         bool endOnEmptyLine = false,
 	                         const std::shared_ptr<ASPeekStream>& streamArg = nullptr) const;
 
@@ -889,6 +880,7 @@ private:  // variables
 	std::string readyFormattedLine;
 	std::string verbatimDelimiter;
 	const std::string* currentHeader;
+	const std::string* previousHeader;
 	char currentChar;
 	char previousChar;
 	char previousNonWSChar;
@@ -905,8 +897,9 @@ private:  // variables
 	int  templateDepth;
 	int  squareBracketCount;
 	int  parenthesesCount;
-	int  squeezeEmptyLineNum;
-	int  squeezeEmptyLineCount;
+	int  closingBracesCount;
+	size_t  squeezeEmptyLineNum;
+	size_t  squeezeEmptyLineCount;
 
 	size_t checksumIn;
 	size_t checksumOut;
@@ -939,13 +932,15 @@ private:  // variables
 	ReferenceAlign referenceAlignment;
 	ObjCColonPad objCColonPadMode;
 	LineEndFormat lineEnd;
+	NegationPaddingMode negationPadMode;
+	IncludeDirectivePaddingMode includeDirectivePaddingMode;
+
 	bool isVirgin;
 	bool isInVirginLine;
 	bool shouldPadCommas;
 	bool shouldPadOperators;
 	bool shouldPadParensOutside;
 	bool shouldPadFirstParen;
-
 	bool shouldPadEmptyParens;
 	bool shouldPadParensInside;
 	bool shouldPadHeader;
@@ -1039,7 +1034,7 @@ private:  // variables
 	bool shouldBreakClosingHeaderBraces;
 	bool shouldBreakElseIfs;
 	bool shouldBreakLineAfterLogical;
-	bool shouldAddBraces;
+	int  shouldAddBraces;
 	bool shouldAddOneLineBraces;
 	bool shouldRemoveBraces;
 	bool shouldPadMethodColon;
@@ -1092,15 +1087,19 @@ private:  // variables
 	bool processedFirstConditional;
 	bool isJavaStaticConstructor;
 	bool isInAllocator;
+	bool isInMultlineStatement;
+	int isInExplicitBlock;
 
 private:  // inline functions
-	// append the CURRENT character (curentChar) to the current formatted line.
+	// append the CURRENT character (currentChar) to the current formatted line.
 	void appendCurrentChar(bool canBreakLine = true)
 	{ appendChar(currentChar, canBreakLine); }
 
 	// check if a specific sequence exists in the current placement of the current line
-	bool isSequenceReached(const char* sequence) const
-	{ return currentLine.compare(charNum, strlen(sequence), sequence) == 0; }
+	bool isSequenceReached(std::string_view sequence) const
+	{
+    	return currentLine.compare(charNum, sequence.length(), sequence) == 0;
+	}
 
 	// call ASBase::findHeader for the current character
 	const std::string* findHeader(const std::vector<const std::string*>* headers_)
